/*
    Copyright (C) 2002 Paul Davis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <iostream>

#include "ardour_dialog.h"
#include "keyboard.h"
#include "ardour_ui.h"
#include "doi.h"

ArdourDialog::ArdourDialog ()
	: Gtk::Window (GTK_WINDOW_TOPLEVEL)
{
	session = 0;
	_within_hiding = false;
}

ArdourDialog::~ArdourDialog ()
{
	EMIT_SIGNAL GoingAway ();
}

gint
ArdourDialog::enter_notify_event_impl (GdkEventCrossing *ev)
{
	if (ev->detail != GDK_NOTIFY_INFERIOR) {
		ARDOUR_UI::instance()->the_keyboard().set_current_dialog (this);
	}
	return FALSE;
}

gint
ArdourDialog::leave_notify_event_impl (GdkEventCrossing *ev)
{
	if (ev->detail != GDK_NOTIFY_INFERIOR) {
		ARDOUR_UI::instance()->the_keyboard().set_current_dialog (0);
	}
	return FALSE;
}

gint
ArdourDialog::unmap_event_impl (GdkEventAny *ev)
{
	_within_hiding = true;
	EMIT_SIGNAL Hiding ();
	_within_hiding = false;
	return Gtk::Window::unmap_event_impl (ev);
}

void
ArdourDialog::wm_close()
{
	EMIT_SIGNAL Hiding ();
	hide_all();
	ARDOUR_UI::instance()->allow_focus(false);
}

void
ArdourDialog::wm_doi ()
{
	hide_all();
	delete_when_idle (this);
}

gint
ArdourDialog::wm_close_event (GdkEventAny* ev)
{
	wm_close ();
	return TRUE;
}

gint
ArdourDialog::wm_doi_event (GdkEventAny* ev)
{
	wm_doi ();
	return TRUE;
}
