/*
    Copyright (C) 1999-2002 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: ardour_ui.cc,v 1.153 2003/06/27 02:19:57 trutkin Exp $
*/

#include <algorithm>
#include <cmath>
#include <fcntl.h>
#include <signal.h>
#include <unistd.h>
#include <cerrno>
#include <fstream>

#include <iostream>

#include <gtk--.h>
#include <pbd/error.h>
#include <pbd/basename.h>
#include <pbd/pathscanner.h>
#include <pbd/failed_constructor.h>
#include <gtkmmext/gtk_ui.h>
#include <gtkmmext/pix.h>
#include <gtkmmext/pixmap_button.h>
#include <gtkmmext/utils.h>
#include <gtkmmext/click_box.h>
#include <gtkmmext/selector.h>
#include <gtkmmext/fastmeter.h>
#include <gtkmmext/stop_signal.h>

#include <midi++/port.h>
#include <midi++/mmc.h>

#include <ardour/ardour.h>
#include <ardour/port.h>
#include <ardour/audioengine.h>
#include <ardour/playlist.h>
#include <ardour/utils.h>
#include <ardour/diskstream.h>
#include <ardour/filesource.h>
#include <ardour/session_diskstream.h>
#include <ardour/port.h>
#include <ardour/audio_track.h>

#include "ardour_ui.h"
#include "editor.h"
#include "audio_clock.h"
#include "keyboard.h"
#include "mixer_ui.h"
#include "prompter.h"
#include "opts.h"
#include "keyboard_target.h"
#include "about.h"

#include "i18n.h"

using namespace ARDOUR;
using namespace Gtkmmext;
using namespace Gtk;

ARDOUR_UI *ARDOUR_UI::theArdourUI = 0;

SigC::Signal1<void,bool> ARDOUR_UI::Blink;
SigC::Signal0<void>      ARDOUR_UI::RapidScreenUpdate;
SigC::Signal1<void,jack_nframes_t> ARDOUR_UI::Clock;

static const gchar *new_session_title[] = {
	N_("Available Mix Templates"),
	0
};

/* XPM */

/* XPM */
static const gchar *h_meter_strip_xpm[] = {
"186 5 77 1",
" 	c None",
".	c #C5CE00",
"+	c #CDCE00",
"@	c #CDCA00",
"#	c #CDC600",
"$	c #CDC200",
"%	c #CDBE00",
"&	c #CDBA00",
"*	c #CDB600",
"=	c #CDB200",
"-	c #CDAE00",
";	c #CDAA00",
">	c #CDA500",
",	c #CDA100",
"'	c #CD9D00",
")	c #CD9900",
"!	c #CD9500",
"~	c #CD9100",
"{	c #CD8D00",
"]	c #CD8900",
"^	c #CD8500",
"/	c #CD8100",
"(	c #CD7D00",
"_	c #CD7900",
":	c #CD7500",
"<	c #CD7100",
"[	c #CD6D00",
"}	c #CD6900",
"|	c #CD6500",
"1	c #CD6100",
"2	c #CD5D00",
"3	c #CD5900",
"4	c #CD5500",
"5	c #CD5000",
"6	c #CD4C00",
"7	c #CD4800",
"8	c #CD4400",
"9	c #CD4000",
"0	c #CD3C00",
"a	c #CD3800",
"b	c #CD3400",
"c	c #CD3000",
"d	c #CD2C00",
"e	c #CD2800",
"f	c #CD2400",
"g	c #CD2000",
"h	c #CD1C00",
"i	c #CD1800",
"j	c #CD1400",
"k	c #CD1000",
"l	c #CD0C00",
"m	c #CD0800",
"n	c #CD0400",
"o	c #00CE00",
"p	c #08CE00",
"q	c #10CE00",
"r	c #18CE00",
"s	c #20CE00",
"t	c #29CE00",
"u	c #31CE00",
"v	c #39CE00",
"w	c #41CE00",
"x	c #4ACE00",
"y	c #52CE00",
"z	c #5ACE00",
"A	c #62CE00",
"B	c #6ACE00",
"C	c #73CE00",
"D	c #7BCE00",
"E	c #83CE00",
"F	c #8BCE00",
"G	c #94CE00",
"H	c #9CCE00",
"I	c #A4CE00",
"J	c #ACCE00",
"K	c #B4CE00",
"L	c #BDCE00",
"                                                                                           ..++@##$$%%&&*==--;;>>,,'))!!~~{{]]^//((__::<[[}}||112234455667788900aabbccdeeffgghhiijkkllmmnn",
" ooooppppqqqrrrrsssstttuuuuvvvvwwwxxxxyyyyzzzAAAABBBBCCCDDDDEEEFFFFGGGGHHHIIIIJJJJKKKLLLL....++@##$$%%&&*==--;;>>,,'))!!~~{{]]^//((__::<[[}}||112234455667788900aabbccdeeffgghhiijkkllmmnn",
" ooooppppqqqrrrrsssstttuuuuvvvvwwwxxxxyyyyzzzAAAABBBBCCCDDDDEEEFFFFGGGGHHHIIIIJJJJKKKLLLL....++@##$$%%&&*==--;;>>,,'))!!~~{{]]^//((__::<[[}}||112234455667788900aabbccdeeffgghhiijkkllmmnn",
" ooooppppqqqrrrrsssstttuuuuvvvvwwwxxxxyyyyzzzAAAABBBBCCCDDDDEEEFFFFGGGGHHHIIIIJJJJKKKLLLL....++@##$$%%&&*==--;;>>,,'))!!~~{{]]^//((__::<[[}}||112234455667788900aabbccdeeffgghhiijkkllmmnn",
" ooooppppqqqrrrrsssstttuuuuvvvvwwwxxxxyyyyzzzAAAABBBBCCCDDDDEEEFFFFGGGGHHHIIIIJJJJKKKLLLL....++@##$$%%&&*==--;;>>,,'))!!~~{{]]^//((__::<[[}}||112234455667788900aabbccdeeffgghhiijkkllmmnn"};

static const gchar *v_meter_strip_xpm[] = {
"5 186 78 1",
" 	c None",
".	c #CD0000",
"+	c #CD0400",
"@	c #CD0800",
"#	c #CD0C00",
"$	c #CD1000",
"%	c #CD1400",
"&	c #CD1800",
"*	c #CD1C00",
"=	c #CD2000",
"-	c #CD2400",
";	c #CD2800",
">	c #CD2C00",
",	c #CD3000",
"'	c #CD3400",
")	c #CD3800",
"!	c #CD3C00",
"~	c #CD4000",
"{	c #CD4400",
"]	c #CD4800",
"^	c #CD4C00",
"/	c #CD5000",
"(	c #CD5500",
"_	c #CD5900",
":	c #CD5D00",
"<	c #CD6100",
"[	c #CD6500",
"}	c #CD6900",
"|	c #CD6D00",
"1	c #CD7100",
"2	c #CD7500",
"3	c #CD7900",
"4	c #CD7D00",
"5	c #CD8100",
"6	c #CD8500",
"7	c #CD8900",
"8	c #CD8D00",
"9	c #CD9100",
"0	c #CD9500",
"a	c #CD9900",
"b	c #CD9D00",
"c	c #CDA100",
"d	c #CDA500",
"e	c #CDAA00",
"f	c #CDAE00",
"g	c #CDB200",
"h	c #CDB600",
"i	c #CDBA00",
"j	c #CDBE00",
"k	c #CDC200",
"l	c #CDC600",
"m	c #CDCA00",
"n	c #CDCE00",
"o	c #C5CE00",
"p	c #BDCE00",
"q	c #B4CE00",
"r	c #ACCE00",
"s	c #A4CE00",
"t	c #9CCE00",
"u	c #94CE00",
"v	c #8BCE00",
"w	c #83CE00",
"x	c #7BCE00",
"y	c #73CE00",
"z	c #6ACE00",
"A	c #62CE00",
"B	c #5ACE00",
"C	c #52CE00",
"D	c #4ACE00",
"E	c #41CE00",
"F	c #39CE00",
"G	c #31CE00",
"H	c #29CE00",
"I	c #20CE00",
"J	c #18CE00",
"K	c #10CE00",
"L	c #08CE00",
"M	c #00CE00",
".....",
"+++++",
"+++++",
"@@@@@",
"@@@@@",
"#####",
"#####",
"$$$$$",
"$$$$$",
"%%%%%",
"&&&&&",
"&&&&&",
"*****",
"*****",
"=====",
"=====",
"-----",
"-----",
";;;;;",
";;;;;",
">>>>>",
",,,,,",
",,,,,",
"'''''",
"'''''",
")))))",
")))))",
"!!!!!",
"!!!!!",
"~~~~~",
"{{{{{",
"{{{{{",
"]]]]]",
"]]]]]",
"^^^^^",
"^^^^^",
"/////",
"/////",
"(((((",
"(((((",
"_____",
":::::",
":::::",
"<<<<<",
"<<<<<",
"[[[[[",
"[[[[[",
"}}}}}",
"}}}}}",
"|||||",
"|||||",
"11111",
"22222",
"22222",
"33333",
"33333",
"44444",
"44444",
"55555",
"55555",
"66666",
"77777",
"77777",
"88888",
"88888",
"99999",
"99999",
"00000",
"00000",
"aaaaa",
"aaaaa",
"bbbbb",
"ccccc",
"ccccc",
"ddddd",
"ddddd",
"eeeee",
"eeeee",
"fffff",
"fffff",
"ggggg",
"ggggg",
"hhhhh",
"iiiii",
"iiiii",
"jjjjj",
"jjjjj",
"kkkkk",
"kkkkk",
"lllll",
"lllll",
"mmmmm",
"nnnnn",
"nnnnn",
"ooooo",
"ooooo",
"ooooo",
"ooooo",
"ppppp",
"ppppp",
"ppppp",
"ppppp",
"qqqqq",
"qqqqq",
"qqqqq",
"rrrrr",
"rrrrr",
"rrrrr",
"rrrrr",
"sssss",
"sssss",
"sssss",
"sssss",
"ttttt",
"ttttt",
"ttttt",
"uuuuu",
"uuuuu",
"uuuuu",
"uuuuu",
"vvvvv",
"vvvvv",
"vvvvv",
"vvvvv",
"wwwww",
"wwwww",
"wwwww",
"xxxxx",
"xxxxx",
"xxxxx",
"xxxxx",
"yyyyy",
"yyyyy",
"yyyyy",
"zzzzz",
"zzzzz",
"zzzzz",
"zzzzz",
"AAAAA",
"AAAAA",
"AAAAA",
"AAAAA",
"BBBBB",
"BBBBB",
"BBBBB",
"CCCCC",
"CCCCC",
"CCCCC",
"CCCCC",
"DDDDD",
"DDDDD",
"DDDDD",
"DDDDD",
"EEEEE",
"EEEEE",
"EEEEE",
"FFFFF",
"FFFFF",
"FFFFF",
"FFFFF",
"GGGGG",
"GGGGG",
"GGGGG",
"GGGGG",
"HHHHH",
"HHHHH",
"HHHHH",
"IIIII",
"IIIII",
"IIIII",
"IIIII",
"JJJJJ",
"JJJJJ",
"JJJJJ",
"JJJJJ",
"KKKKK",
"KKKKK",
"KKKKK",
"LLLLL",
"LLLLL",
"LLLLL",
"LLLLL",
"MMMMM",
"MMMMM",
"MMMMM",
"MMMMM"};


ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[], string rcfile)

	: Gtkmmext::UI ("ardour", argcp, argvp, rcfile),

	  primary_clock (X_("TransportClockDisplay"), true),
	  secondary_clock (X_("SecondaryClockDisplay"), true),

	  /* adjuster table */

	  adjuster_table (3, 3),

	  /* preroll stuff */

	  preroll_button (_("pre\nroll")),
	  postroll_button (_("post\nroll")),
	  preroll_clock (X_("PreRollClock"), true, true),
	  postroll_clock (X_("PostRollClock"), true, true),

	  /* clock window */

	  big_clock ("BigClockDisplay", true),

	  /* transport */
	  
	  punch_in_button ("punch\nin"),
	  punch_out_button ("punch\nout"),
	  auto_return_button ("auto\nreturn"),
	  auto_play_button ("auto\nplay"),
	  auto_input_button ("auto\ninput"),
	  click_button ("click"),

	  session_selector (1, 0),
	  
	  wipe_level (0),

	  /* new session dialog */
	  template_selector (_mix_template_refiller, this, internationalize (new_session_title), false, true),

	  /* nudge */

	  nudge_label (_("(Nudge)")),
	  nudge_clock ("NudgeClock", true, true)

{
	/* actually, its already loaded, but ... */

	cerr << "Loading UI configuration file " << rcfile << endl;

	about = 0;

	if (theArdourUI == 0) {
		theArdourUI = this;
	}

	editor = 0;
	mixer = 0;
	connection_editor = 0;
	add_route_dialog = 0;
	route_params = 0;
	meter_bridge = 0;
	option_editor = 0;
	location_ui = 0;
	sfdb_window = 0;
	wait_for_loading_window = 0;

	/* have to wait for AudioEngine and Configuration before proceeding */
}

void
ARDOUR_UI::set_engine (AudioEngine& e)
{
	engine = &e;
	engine->Halted.connect (slot (*this, &ARDOUR_UI::engine_halted));
	engine->SampleRateChanged.connect (slot (*this, &ARDOUR_UI::update_sample_rate));

	_tooltips.enable();

	_global_settings = Config->extra_xml("global-gtk");

	if (_global_settings == 0){
		init_global_settings();
	}

	keyboard = new Keyboard;
 	install_keybindings ();

	FastMeter::set_vertical_xpm (v_meter_strip_xpm);
	FastMeter::set_horizontal_xpm (h_meter_strip_xpm);

	if (setup_windows ()) {
		throw failed_constructor ();
	}

	if (GTK_ARDOUR::show_key_actions) {
		KeyboardTarget::show_all_actions ();
		exit (0);
	}

	/* start with timecode, metering enabled
	*/
	
	blink_timeout_tag = -1;

	/* this being a GUI and all, we want peakfiles */

	FileSource::set_build_peakfiles (true);
	FileSource::set_build_missing_peakfiles (true);

	if (Source::start_peak_thread ()) {
		throw failed_constructor();
	}

	session = 0;
	session_selector_window = 0;
	new_session_window = 0;
	last_key_press_time = 0;
	
	/* start the time-of-day-clock */
	
	update_wall_clock ();
	Main::timeout.connect (slot (this, &ARDOUR_UI::update_wall_clock), 60000);

	update_disk_space ();
	update_cpu_load ();
	update_sample_rate (engine->frame_rate());

	starting.connect (slot (*this, &ARDOUR_UI::startup));
	stopping.connect (slot (*this, &ARDOUR_UI::shutdown));
}

ARDOUR_UI::~ARDOUR_UI ()
{
	if (keyboard) {
		_global_settings->remove_nodes ("Keyboard");
		_global_settings->add_child_nocopy (keyboard->get_state());
		delete keyboard;
	}

	if (editor) {
		_global_settings->remove_nodes ("Editor");
		_global_settings->add_child_nocopy (editor->get_state());
		delete editor;
	}

	if (mixer) {
		_global_settings->remove_nodes ("Mixer");
		_global_settings->add_child_nocopy (mixer->get_state());
		delete mixer;
	}

	Source::stop_peak_thread ();

	Config->add_extra_xml (*_global_settings);
	Config->save_state();
}

void
ARDOUR_UI::init_global_settings()
{
	_global_settings = new XMLNode("global-gtk");
	_global_settings->add_property("editor_x_size", "995");
	_global_settings->add_property("editor_y_size", "765");
	_global_settings->add_property("editor_x_pos", "1");
	_global_settings->add_property("editor_y_pos", "1");
	_global_settings->add_property("editor_x_off", "0");
	_global_settings->add_property("editor_y_off", "21");
	_global_settings->add_property("mixer_x_size", "-1");
	_global_settings->add_property("mixer_y_size", "765");
	_global_settings->add_property("mixer_x_pos", "1");
	_global_settings->add_property("mixer_y_pos", "1");
	_global_settings->add_property("mixer_x_off", "0");
	_global_settings->add_property("mixer_y_off", "21");
}

void
ARDOUR_UI::startup ()
{
	/* Once the UI is up and running, start the audio engine. Doing
	   this before the UI is up and running can cause problems
	   when not running with SCHED_FIFO, because the amount of
	   CPU and disk work needed to get the UI started can interfere
	   with the scheduling of the audio thread.
	*/

	Gtk::Main::idle.connect (slot (*this, &ARDOUR_UI::start_engine));
}

void
ARDOUR_UI::finish()
{
	if (session && session->dirty()) {
		switch (ask_about_saving_session(_("quit"))) {
		case -1:
			return;
			break;
		case 1:
			session->save_state ("default");
			break;
		case 0:
			break;
		}
	}

	quit();
}

int
ARDOUR_UI::ask_about_saving_session (string what)
{
	ArdourDialog window;
	Gtk::VBox   packer;
	Gtk::Label  prompt_label;
	Gtk::HBox   button_packer;

	string msg;

	msg = compose(_("Save and %1"), what);
	
	Gtk::Button save_button (msg);

	msg = compose(_("Just %1"), what);

	Gtk::Button nosave_button (msg);

	msg = compose(_("Don't %1"), what);

	Gtk::Button noquit_button (msg);

	string prompt;
	prompt = compose(_("The session \"%1\"\nhas not been saved.\n\nAny changes made this time\nwill be lost unless you save it.\n\nWhat do you want to do?"), session->name());
	
	prompt_label.set_text (prompt);
	prompt_label.set_alignment (0.5, 0.5);
	prompt_label.set_name ("PrompterLabel");

	save_button.clicked.connect (bind(slot(*this,&ARDOUR_UI::save_session_choice_made), 1));
	nosave_button.clicked.connect (bind(slot(*this,&ARDOUR_UI::save_session_choice_made), 0));
	noquit_button.clicked.connect (bind(slot(*this,&ARDOUR_UI::save_session_choice_made), -1));
	window.delete_event.connect (slot(*this,&ARDOUR_UI::ask_about_save_deleted));

	button_packer.set_spacing (10);
	button_packer.pack_start (save_button);
	button_packer.pack_start (nosave_button);
	button_packer.pack_start (noquit_button);
	
	packer.set_spacing (10);
	packer.set_border_width (10);
	packer.pack_start (prompt_label);
	packer.pack_start (button_packer);

	window.set_name ("Prompter");
	window.set_title (_("ardour: save session?"));
	window.set_position (GTK_WIN_POS_MOUSE);
	window.set_modal (true);
	window.add (packer);
	window.show_all ();

	window.realize();
	window.get_window().set_decorations (GdkWMDecoration (GDK_DECOR_BORDER|GDK_DECOR_RESIZEH));

	keyboard->allow_focus (true);

	save_the_session = 0;

	Main::run ();
	
	keyboard->allow_focus (false);

	return save_the_session;
}
	
void
ARDOUR_UI::save_session_choice_made (int result)
{
	save_the_session = result;
	Main::quit ();
}

gint
ARDOUR_UI::ask_about_save_deleted (GdkEventAny *ignored)
{
	save_the_session = -1;
	Main::quit ();
	return TRUE;
}

gint
ARDOUR_UI::every_second ()
{
	update_cpu_load ();
	update_buffer_load ();
	update_disk_space ();
	update_disk_rate ();
	return TRUE;
}

gint
ARDOUR_UI::every_point_one_seconds ()
{
	EMIT_SIGNAL IO::GrabPeakPower();
	EMIT_SIGNAL RapidScreenUpdate();
	return TRUE;
}

void
ARDOUR_UI::update_sample_rate (jack_nframes_t nframes)
{
	char buf[32];

	if (fmod (nframes, 1000.0) != 0.0) {
		snprintf (buf, sizeof (buf), _("SR: %.1f kHz"), (float) nframes/1000.0f);
	} else {
		snprintf (buf, sizeof (buf), _("SR: %lu kHz"), nframes/1000);
	}

	sample_rate_label.set_text (buf);
}

void
ARDOUR_UI::update_cpu_load ()
{
	char buf[32];
	snprintf (buf, sizeof (buf), _("CPU Load: %.1f%%"), engine->get_cpu_load());
	cpu_load_label.set_text (buf);
}

void
ARDOUR_UI::update_disk_rate ()
{
	char buf[64];

	if (session) {
		snprintf (buf, sizeof (buf), _("Disk r:%5.1f w:%5.1f MB/s"), 
			  session->read_data_rate()/1048576.0f, session->write_data_rate()/1048576.0f);
		disk_rate_label.set_text (buf);
	} else {
		disk_rate_label.set_text ("");
	}
}

void
ARDOUR_UI::update_buffer_load ()
{
	char buf[64];

	if (session) {
		snprintf (buf, sizeof (buf), _("Buffers p:%5.0f%% c:%5.0f%%"), 
			  session->playback_load() * 100.0f, 
			  session->capture_load() * 100.0f);
		buffer_load_label.set_text (buf);
	} else {
		buffer_load_label.set_text ("");
	}
}

void
ARDOUR_UI::count_recenabled_diskstreams (DiskStream& ds)
{
	if (ds.record_enabled()) {
		rec_enabled_diskstreams++;
	}
}

void
ARDOUR_UI::update_disk_space()
{
	if (session == 0) {
		return;
	}

	jack_nframes_t frames = session->available_capture_duration();
	char buf[64];

	if (frames == max_frames) {
		strcpy (buf, _("space: 24hrs+"));
	} else {
		int hrs;
		int mins;
		int secs;
		jack_nframes_t fr = session->frame_rate();
		
		if (session->actively_recording()){
			
			rec_enabled_diskstreams = 0;
			session->foreach_diskstream (this, &ARDOUR_UI::count_recenabled_diskstreams);
			
			if (rec_enabled_diskstreams) {
				frames /= rec_enabled_diskstreams;
			}
			
		} else {
			
			/* hmmm. shall we divide by the route count? or the diskstream count?
			   or what? for now, do nothing ...
			*/
			
		}
		
		hrs  = frames / (fr * 3600);
		frames -= hrs * fr * 3600;
		mins = frames / (fr * 60);
		frames -= mins * fr * 60;
		secs = frames / fr;
		
		snprintf (buf, sizeof(buf), _("space: %02dh:%02dm:%02ds"), hrs, mins, secs);
	}

	disk_space_label.set_text (buf);
}		  

gint
ARDOUR_UI::update_wall_clock ()
{
	time_t now;
	struct tm *tm_now;
	char buf[16];

	time (&now);
	tm_now = localtime (&now);

	sprintf (buf, "%02d:%02d", tm_now->tm_hour, tm_now->tm_min);
	wall_clock_label.set_text (buf);

	return TRUE;
}

int
ARDOUR_UI::setup_pixmaps ()
{
	int retval = -1;
	string path;
	char *envvar;

	if ((envvar = getenv ("HOME")) != 0) {
		path = envvar;
		path += "/.ardour/pixmaps:";
	} 

	path += DATA_DIR;
	path += "/ardour/pixmaps";

	if ((toggle_buttons = get_pix (path, "^toggle-button-[0-9]*.xpm$")) == 0) {
		error << _("No toggle button pixmaps found to match toggle-button-[0-9]*.xpm$") << endmsg;
		goto headback;
	}

	toggle_buttons->ref ();
	
	if ((small_push_buttons = get_pix (path, "^small-round-button-[0-9]*.xpm$")) == 0) {
		error << _("No small push button pixmaps found to match small-round-button-[0-9]*.xpm$") << endmsg;
		goto headback;
	}

	small_push_buttons->ref ();

	if ((plugin_ui_sliders = get_pix (path, "^hslider[0-9]*.xpm$")) == 0) {
		error << _("No pixmaps found to match hslider[0-9]*.xpm$") << endmsg;
		goto headback;
	}

	plugin_ui_sliders->ref ();

	if ((mixer_sliders = get_pix (path, "^vslider[0-9]*.xpm$")) == 0) {
		error << _("No pixmaps found to match vslider[0-9]*.xpm$") << endmsg;
		goto headback;
	}

	mixer_sliders->ref ();

	retval = 0;

  headback:
	return retval;
}

#define ReferenceAndReturn(x) \
           Pix *ARDOUR_UI::x ## _pix () { x ## s->ref(); return x ## s; }

ReferenceAndReturn(small_push_button)
ReferenceAndReturn(toggle_button)
ReferenceAndReturn(plugin_ui_slider)
ReferenceAndReturn(mixer_slider)

void
ARDOUR_UI::toggle_recording_plugins ()

{
	/* XXX use toggle_some_session_state */

	if (session == 0) {
		return;
	}

	session->set_recording_plugins (!session->get_recording_plugins());
}
	
void
ARDOUR_UI::toggle_auto_play ()

{
	toggle_some_session_state (auto_play_button,
				   &Session::get_auto_play,
				   &Session::set_auto_play);
}

void
ARDOUR_UI::toggle_auto_return ()

{
	toggle_some_session_state (auto_return_button,
				   &Session::get_auto_return,
				   &Session::set_auto_return);
}

void
ARDOUR_UI::toggle_click ()
{
	toggle_some_session_state (click_button,
				   &Session::get_clicking,
				   &Session::set_clicking);
}

void
ARDOUR_UI::toggle_auto_loop ()
{
	toggle_some_session_state (auto_loop_button,
				   &Session::get_auto_loop,
				   &Session::request_auto_loop);
}
  
void
ARDOUR_UI::toggle_punch_out ()
{
	toggle_some_session_state (punch_out_button,
				   &Session::get_punch_out,
				   &Session::set_punch_out);
}

void
ARDOUR_UI::toggle_punch_in ()
{
	toggle_some_session_state (punch_in_button,
				   &Session::get_punch_in,
				   &Session::set_punch_in);
}

void
ARDOUR_UI::map_button_state ()

{
	map_some_session_state (auto_return_button,
				&Session::get_auto_return);
	map_some_session_state (auto_play_button,
				&Session::get_auto_play);
	map_some_session_state (auto_input_button,
				&Session::get_auto_input);
	map_some_session_state (auto_loop_button,
				&Session::get_auto_loop);
	map_some_session_state (punch_in_button,
				&Session::get_punch_in);
	map_some_session_state (punch_out_button,
				&Session::get_punch_out);
	map_some_session_state (click_button,
				&Session::get_clicking);
}

void
ARDOUR_UI::queue_map_control_change (Session::ControlType t)
{
	call_slot (bind (slot (*this, &ARDOUR_UI::map_control_change), t));
}

void
ARDOUR_UI::map_control_change (Session::ControlType t)
{
	switch (t) {
	case Session::AutoPlay:
		map_some_session_state (auto_play_button, &Session::get_auto_play);
		break;

	case Session::AutoLoop:
		map_some_session_state (auto_loop_button, &Session::get_auto_loop);
		break;

	case Session::AutoReturn:
		map_some_session_state (auto_return_button, &Session::get_auto_return);
		break;

	case Session::AutoInput:
		map_some_session_state (auto_input_button, &Session::get_auto_input);
		break;

	case Session::PunchOut:
		map_some_session_state (punch_in_button, &Session::get_punch_out);
		break;

	case Session::PunchIn:
		map_some_session_state (punch_in_button, &Session::get_punch_in);
		break;

	case Session::Clicking:
		map_some_session_state (click_button, &Session::get_clicking);
		break;

	case Session::SlaveType:
//		map_some_session_state (mtc_slave_button, &Session::get_mtc_slave);
		break;

	case Session::SendMTC:
//		map_some_session_state (send_mtc_button, &Session::get_send_mtc);
		break;

	case Session::SendMMC:
//		map_some_session_state (send_mmc_button, &Session::get_send_mmc);
		break;

	case Session::MMCControl:       
//		map_some_session_state (mmc_control_button, &Session::get_mmc_control);
		break;

	case Session::Live:
		break;

	case Session::RecordingPlugins:
		break;

	case Session::AutoCrossFade:
		break;
		
	case Session::EditingMode:
		break;

	case Session::PlayRange:
		map_some_session_state (play_selection_button, &Session::get_play_range);
		break;

	}
}

void
ARDOUR_UI::control_methods_adjusted ()

{
	int which_method;

	which_method = (int) online_control_button->adjustment.get_value();
	switch (which_method) {
	case 0:
		allow_mmc_and_local ();
		break;
	case 1:
		allow_mmc_only ();
		break;
	case 2:
		allow_local_only ();
		break;
	default:
		fatal << _("programming error: impossible control method") << endmsg;
	}
}
	

void
ARDOUR_UI::mmc_device_id_adjusted ()

{
#if 0
	if (mmc) {
		int dev_id = (int) mmc_id_button->adjustment.get_value();
		mmc->set_device_id (dev_id);
	}
#endif
}

void
ARDOUR_UI::map_some_session_state (PixmapButton *button,
				   bool (Session::*get)() const)
	
{
	if (session == 0) {
		return;
	}
	
	button->set_active ((session->*get)());
}

void
ARDOUR_UI::map_some_session_state (ToggleButton& button,
				   bool (Session::*get)() const)
	
{
	bool x;

	if (session == 0) {
		return;
	}
	
	if (button.get_active() != (x = (session->*get)())) {
		button.set_active (x);
	}
}

void
ARDOUR_UI::toggle_some_session_state (PixmapButton *button,
				      bool (Session::*get)() const,
				      void (Session::*set)(bool))

{
	bool button_state;
	bool session_state;

	if (session == 0) {
		return;
	}

	button_state = button->get_active ();
	session_state = (session->*get)();

	if (button_state != session_state) {
		(session->*set) (button_state);
#if 0
	
		/* check that it worked, and reverse
		   the button state if it didn't
		*/

		if ((session->*get)() != button_state) {
			button->set_active (!button_state);
		}
#endif

	}
}	

void
ARDOUR_UI::toggle_some_session_state (ToggleButton& button,
				      bool (Session::*get)() const,
				      void (Session::*set)(bool))

{
	bool button_state;
	bool session_state;

	if (session == 0) {
		return;
	}

	button_state = button.get_active ();
	session_state = (session->*get)();

	if (button_state != session_state) {
		(session->*set) (button_state);
#if 0
	
		/* check that it worked, and reverse
		   the button state if it didn't
		*/

		if ((session->*get)() != button_state) {
			button->set_active (!button_state);
		}
#endif

	}
}	

int
ARDOUR_UI::load_session (string name, string snap_name, string* mix_template)

{
	Session *new_session;
	string fullpath;
	bool isnew;
	bool had_session = (session != 0);

	unload_session ();
	
	if (had_session) {
		show_wait_for_loading();
	}

	fullpath = Session::find_session (name, isnew);

	if (fullpath.length() == 0) {
		error << compose (_("cannot find a session named \"%1\""), name) << endmsg;
		return -1;
	}

	try {
		new_session = new Session (*engine, fullpath, name, snap_name, mix_template);
	}

	catch (...) {

		hide_wait_for_loading ();

		error << compose(_("Session \"%1\" did not load successfully"), name) << endmsg;
		return -1;
	}

	connect_to_session (new_session);

	if (engine->running()) {
		hide_wait_for_loading ();

		mixer->show_window();
	}
	
	return 0;
}

gint
ARDOUR_UI::session_menu (GdkEventButton *ev)
{
	session_popup_menu->popup (0, 0, 0, 0);
	return TRUE;
}

static bool 
session_filter (const string &str, void *arg)
{
	struct stat statbuf;
	string default_state;

	/* check that it exists */

	if (stat (str.c_str(), &statbuf)) {
		return false;
	}

	/* check that its a directory */

	if (!S_ISDIR(statbuf.st_mode)) {
		return false;
	}

	/* check that a default session state exists */

	default_state = str;
	default_state += "/default";
	default_state += Session::statefile_suffix();
	
	if (stat (default_state.c_str(), &statbuf)) {
		return false;
	}

	/* check that the statefile is a regular file */

	if (!S_ISREG (statbuf.st_mode)) {
		return false;
	}

	/* check that the statefile appears to contain XML */

	ifstream in (default_state.c_str());
	
	if (!in) {
		return false;
	}
	
	string first;

	in >> first;

	if (first != "<?xml") {
		return false;
	}
	
	return true;
}	

struct string_cmp {
    bool operator()(const string* a, const string* b) {
	    return *a < *b;
    }
};

void
ARDOUR_UI::redisplay_all_sessions () 
{
	redisplay_some_sessions (true);
}

void
ARDOUR_UI::redisplay_recent_sessions ()
{
	redisplay_some_sessions (false);
}

void
ARDOUR_UI::redisplay_some_sessions (bool all)
{
	using namespace Gtkmmext;
	using namespace Gtk::CTree_Helpers;

	vector<string *> *sessions;
	vector<string *>::iterator i;
	PathScanner scanner;
	string_cmp cmp;

	/* ---------------------------------------- */
	/* XXX MAKE ME A FUNCTION (no CTree::clear() in gtkmm 1.2) */

	gtk_ctree_remove_node (session_selector.gtkobj(), NULL);

	/* ---------------------------------------- */

	if (all) {
		
		if ((sessions = scanner (Config->get_session_path(), session_filter, 0, true, true)) == 0) {
			return;
		}

		sort (sessions->begin(), sessions->end(), cmp);
		
		if (sessions->size() == 0) {
			delete sessions;
			error << compose(_("No sessions found using search path \"%1\""), Config->get_session_path()) << endmsg;
			session_selector.thaw();
			return;
		}

	} else {

		Session::RecentSessions rs;
		Session::read_recent_sessions (rs);

		if (rs.empty()) {
			error << _("No recent sessions known") << endmsg;
			session_selector.thaw();
			return;
		}

		sessions = new vector<string*>;

		for (Session::RecentSessions::iterator i = rs.begin(); i != rs.end(); ++i) {
			sessions->push_back (new string ((*i).second));
		}
	}

	session_selector.freeze();

	for (i = sessions->begin(); i != sessions->end(); ++i) {

		vector<string*>* states;
		string fullpath = *(*i);
		string::size_type start, end;

		/* remove any trailing / */

		if (fullpath[fullpath.length()-1] == '/') {
			fullpath = fullpath.substr (0, fullpath.length()-1);
		}

		/* now get available states for this session */

		if ((states = Session::possible_states(fullpath)) == 0) {
			/* no state file? */
			continue;
		}

		/* OK, try to add entries for this session */

		sort (states->begin(), states->end(), cmp);

		start = fullpath.find_last_of ('/') + 1;

		if ((end = fullpath.find_last_of ('.')) == string::npos) {
			end = fullpath.length();
		}

		vector<const gchar*> item;

		if (states->size() == 1) {

			/* Only one state file, so don't create a subtree */
			
			item.clear ();
			item.push_back (fullpath.substr (start, (end-start)).c_str());
			session_selector.rows().push_back (Element (item));
			session_selector.rows().back().set_data (*i, deferred_delete<string>);	
			session_selector.rows().back().set_leaf (true);
			

		} else {
			
			/* add the parent */

			string foo = fullpath.substr (start, (end-start));
			item.clear ();
			item.push_back (foo.c_str());
			session_selector.rows().push_back (Element (item));
			session_selector.rows().back().set_data (*i, deferred_delete<string>);
			session_selector.rows().back().set_leaf (false);

			vector<string *>::iterator i2;

			/* add the children */

			for (i2 = states->begin(); i2 != states->end(); ++i2) {
	
				string statename = *(*i2);

				if ((start = statename.find_last_of ('/')) != string::npos) {
					statename = statename.substr (start+1);
				}
				
				if ((end = statename.find_last_of ('.')) == string::npos) {
					end = statename.length();
				}

				statename = statename.substr (0, end);

				item.clear ();
				item.push_back (statename.c_str());
				
				session_selector.rows().back().subtree().push_back (Element (item));
				session_selector.rows().back().subtree().back().set_data (new string (statename),
											  deferred_delete<string>);
				session_selector.rows().back().subtree().back().set_leaf (true);
				
				delete *i2;
			}
		}

		delete states;
	}

	session_selector.thaw();
	delete sessions;
}

void
ARDOUR_UI::session_selection (Gtk::CTree_Helpers::Row row, gint column)
{
	using namespace Gtk::CTree_Helpers;

	string session_name;
	string session_path;
	string session_state;

	if (!row.is_leaf()) {
		row.expand();
		return;
	}

	string *stp = static_cast<string *> (row.get_data());

	if ((*stp)[0] != '/' && (*stp)[0] != '.') {

		/* its a state file node, so get the parent for the session information,
		   and combine with the state file name.
		*/
		
		string *spp = static_cast<string *> (row.get_parent().get_data());
		
		session_name = *spp;
		session_path = *spp;
		session_state = *stp;
		
	} else {

		/* its a session directory node, so just get the session path,
		   and use "default" to load the state.
		*/
		
		string *spp = static_cast<string *> (row.get_data());

		session_name = *spp;
		session_path = *spp;
		session_state = "default";
	}

	session_selector_window->hide ();
	load_session (session_name, session_state);
}

void
ARDOUR_UI::build_session_selector ()
{
	session_selector_window = new ArdourDialog;
	
	Gtk::VBox *vpacker = new Gtk::VBox;
	Gtk::HBox *button_packer = new Gtk::HBox;
	Gtk::Button *cancel_button = new Gtk::Button (_("cancel"));
	Gtk::Button *rescan_button = new Gtk::Button (_("rescan"));

	button_packer->pack_start (*rescan_button);
	button_packer->pack_start (*cancel_button);

	vpacker->pack_start (session_selector);
	vpacker->pack_start (*button_packer, false, false);

	session_selector_window->add (*vpacker);
	session_selector_window->set_name ("SessionSelectorWindow");
	session_selector_window->set_usize (200, 400);

	session_selector_window->delete_event.connect (bind (slot (Gtkmmext::UI::just_hide_it), static_cast<Gtk::Window*>(session_selector_window)));
	cancel_button->clicked.connect (bind (slot (*this, &ARDOUR_UI::hide_dialog), session_selector_window));
	rescan_button->clicked.connect (slot (*this, &ARDOUR_UI::redisplay_all_sessions));
	session_selector.tree_select_row.connect (slot (*this, &ARDOUR_UI::session_selection));
}

void
ARDOUR_UI::open_session ()
{
	/* popup selector window */

	if (session_selector_window == 0) {
		build_session_selector ();
	}

	redisplay_all_sessions ();
	session_selector_window->show_all ();

	/* wait for selection */
}

void
ARDOUR_UI::open_recent_session ()
{
	/* popup selector window */

	if (session_selector_window == 0) {
		build_session_selector ();
	}

	redisplay_recent_sessions ();
	session_selector_window->show_all ();

	/* wait for selection */
}

struct ARDOUR_UI::AddRouteDialog : public ArdourDialog {

    AddRouteDialog (string what);
    ~AddRouteDialog () {}

    SigC::Signal1<void,int> Done;

    Frame       frame;
    Label       spin_label;
    Adjustment  adjustment;
    SpinButton  spinbutton;
    HBox        spin_box;
    Button      ok_button;
    Button      cancel_button;
    HBox        button_box;
    VBox        vpacker;

    gint deleted (GdkEventAny *);
    void cancelled ();
    void done ();
};

ARDOUR_UI::AddRouteDialog::AddRouteDialog (string what)
	: adjustment (1, 1, 128, 1, 4, 0),
	  spinbutton (adjustment),
	  ok_button (_("OK")),
	  cancel_button (_("Cancel"))
{
	spin_label.set_text (string(compose(_("Number of %1"), what)));

	spin_box.set_border_width (5);
	spin_box.pack_start (spin_label, false, false);
	spin_box.pack_start (spinbutton, false, false);

	button_box.set_border_width (5);
	button_box.set_homogeneous (true);
	button_box.pack_start (ok_button, false, false);
	button_box.pack_start (cancel_button, false, false);

	vpacker.set_border_width (5);
	vpacker.pack_start (spin_box, false, true);
	vpacker.pack_start (button_box, false, true);

	frame.set_name ("BaseFrame");
	frame.add (vpacker);

	set_name ("RouteDialogWindow");
	add (frame);

	delete_event.connect (slot (this, &AddRouteDialog::deleted));
	cancel_button.clicked.connect (slot (this, &AddRouteDialog::cancelled));
	ok_button.clicked.connect (slot (this, &AddRouteDialog::done));
}

void
ARDOUR_UI::AddRouteDialog::done ()
{
	EMIT_SIGNAL Done (1);
}

void
ARDOUR_UI::AddRouteDialog::cancelled ()
{
	EMIT_SIGNAL Done(-1);
}

gint
ARDOUR_UI::AddRouteDialog::deleted (GdkEventAny *ev)
{
	EMIT_SIGNAL Done(-1);
	return TRUE;
}

void
ARDOUR_UI::session_add_midi_track ()
{
	cerr << _("Patience is a virtue.\n");
}

void
ARDOUR_UI::session_add_audio_route (bool disk, int input_channels, int output_channels)
{
	Route* route;

	if (session == 0) {
		warning << _("You cannot add a track without a session already loaded.") << endmsg;
		return;
	}

	if (disk) {
		if ((route = session->new_audio_track (input_channels, output_channels)) == 0) {
			error << _("could not create new audio track") << endmsg;
		}
	} else {
		if ((route = session->new_audio_route (input_channels, output_channels)) == 0) {
			error << _("could not create new audio bus") << endmsg;
		}
	}
	
#if CONTROLOUTS
	if (need_control_room_outs) {
		pan_t pans[2];
		
		pans[0] = 0.5;
		pans[1] = 0.5;
		
		route->set_stereo_control_outs (control_lr_channels);
		route->control_outs()->set_stereo_pan (pans, this);
	}
#endif /* CONTROLOUTS */
}

void
ARDOUR_UI::diskstream_added (DiskStream& ds)
{
	meter_bridge_dialog_check->set_sensitive (true);
}

void
ARDOUR_UI::do_transport_locate (jack_nframes_t new_position)
{
	jack_nframes_t _preroll;

	if (session) {
		_preroll = session->convert_to_frames_at (new_position, session->preroll);

		if (new_position > _preroll) {
			new_position -= _preroll;
		} else {
			new_position = 0;
		}

		session->request_locate (new_position);
	}
}

void
ARDOUR_UI::transport_locate ()
{
	if (session) {
		Location *loc = session->locations()->current ();
		if (loc) {
			do_transport_locate (loc->start());
		} else {
			do_transport_locate (0);
		}
	}
}

void
ARDOUR_UI::transport_goto_start ()
{
	if (session) {
		session->request_locate (0);
	}
}

void
ARDOUR_UI::transport_goto_end ()
{
	if (session) {
		session->request_locate (session->current_end_frame());
	}
}

gint 
ARDOUR_UI::mouse_transport_stop (GdkEventButton *ev)
{
	if (session) {
		if (session->transport_stopped()) {
			session->request_locate (session->last_transport_start());
		} else {
			Keyboard::ModifierMask mask = Keyboard::ModifierMask (Keyboard::Control|Keyboard::Alt);
			session->request_stop (Keyboard::modifier_state_equals (ev->state, mask));
		}
	}

	return TRUE;
}
gint
ARDOUR_UI::mouse_transport_roll (GdkEventButton* ev)
{
	if (session) {
		if (session->transport_rolling()) {
			session->request_locate (session->last_transport_start(), true);
		} else {
			session->request_roll ();
		}
	}

	return TRUE;
}

void
ARDOUR_UI::transport_stop ()
{
	if (session) {
		session->request_stop ();
	}
}

void
ARDOUR_UI::transport_stop_and_forget_capture ()
{
	if (session) {
		session->request_stop (true);
	}
}

void
ARDOUR_UI::transport_record ()
{
	if (session) {
		switch (session->record_status()) {
		case Session::Disabled:
			session->maybe_enable_record ();
			break;
		case Session::Recording:
		case Session::Enabled:
			session->disable_record ();
		}
	}
}

void
ARDOUR_UI::transport_roll ()
{
	if (session) {
		session->request_roll ();
	}
}

void
ARDOUR_UI::play_selection_toggled ()
{
	if (session == 0) {
		return;
	}

	bool x = session->get_play_range();

	if (play_selection_button.get_active () != x) {
		session->request_play_range (!x);
	}
}

void
ARDOUR_UI::transport_play_selection ()
{
	/* how to unset it? */
	session->request_play_range (true);
}

void
ARDOUR_UI::transport_rewind ()
{
	float current_transport_speed;
 
       	if (session) {
		current_transport_speed = session->transport_speed();
		
		if (current_transport_speed > 0.0f) {
			/* reset to regular reverse speed */
			session->request_transport_speed (-1.0f);
		} else if (current_transport_speed == 0.0f) {
			session->request_transport_speed (-1.2f);
		} else {
			/* speed up */
			session->request_transport_speed (current_transport_speed * 1.2f);
		}
	}
}

void
ARDOUR_UI::transport_forward ()
{
	float current_transport_speed;
	
	if (session) {
		current_transport_speed = session->transport_speed();
		
		if (current_transport_speed < 0.0f) {
			/* reset to regular forward speed */
			session->request_transport_speed (1.0f);
		} else if (current_transport_speed == 0.0f) {
			session->request_transport_speed (1.2f);
		} else {
			/* speed up */
			session->request_transport_speed (current_transport_speed * 1.2f);
		}
	}
}

void
ARDOUR_UI::toggle_monitor_enable (guint32 dstream)
{
	if (session == 0) {
		return;
	}

	DiskStream *ds;

	if ((ds = session->diskstream_by_id (dstream)) != 0) {
		Port *port = ds->input_port();
		port->request_monitor_input (!port->monitoring_input());
	}
}

void
ARDOUR_UI::toggle_record_enable (guint32 dstream)
{
	if (session == 0) {
		return;
	}

	DiskStream *ds;

	if ((ds = session->diskstream_by_id (dstream)) != 0) {
		ds->set_record_enabled (!ds->record_enabled(), this);
	}
}

void
ARDOUR_UI::queue_transport_change ()
{
	Gtkmmext::UI::instance()->call_slot (slot (*this, &ARDOUR_UI::map_transport_state));
}

void
ARDOUR_UI::map_transport_state ()
{
	float sp = session->transport_speed();

	if (sp == 1.0f) {
		transport_rolling ();
	} else if (sp < 0.0f) {
		transport_rewinding ();
	} else if (sp > 0.0f) {
		transport_forwarding ();
	} else {
		transport_stopped ();
	}
}

void
ARDOUR_UI::allow_local_only ()
{

}

void
ARDOUR_UI::allow_mmc_only ()
{

}

void
ARDOUR_UI::allow_mmc_and_local ()
{

}

void
ARDOUR_UI::wipe_session ()
{
	if (session == 0) {
		return;
	}

	switch (wipe_level) {
	case 0:
		wipe_window.show_all ();
		wipe_window.set_title (_("Wipe"));
		wipe_commit_label.set_text (_("Fill with silence ? (non-destructive)"));
		wipe_level++;
		break;

	case 1:
		wipe_window.hide_all ();
		// session->wipe ();
		wipe_level = 0;
	}
}

void
ARDOUR_UI::wipe_cancel ()
{
	wipe_window.hide_all ();
	wipe_level = 0;
}

int
ARDOUR_UI::wipe_delete (GdkEventAny *ev)
{
	wipe_cancel ();
	return TRUE;
}

void
ARDOUR_UI::GlobalClickBox::printer (char buf[32], Adjustment &adj, void *arg)
{
	snprintf (buf, sizeof(buf), "%s", ((GlobalClickBox *) arg)->strings[
		(int) adj.get_value()].c_str());
}

void
ARDOUR_UI::engine_halted ()
{
	Gtkmmext::UI::instance()->call_slot (slot (*this, &ARDOUR_UI::notify_engine_halted));
}

void
ARDOUR_UI::notify_engine_halted ()
{
	ArdourDialog *window = new ArdourDialog;
	Label *label;
	VBox *vbox = manage (new VBox);
	HBox *hbox = manage (new HBox);
	// HBox *button_hbox = manage (new HBox);
	
	label = manage (new Label);
	
	label->set_text (_("JACK has either been shutdown or it\ndisconnected Ardour because Ardour\nwas not fast enough. You should save the\nsession and restart both JACK and Ardour."));

	vbox->set_border_width (10);
	hbox->set_border_width (10);
	vbox->pack_start (*label);
	hbox->pack_start (*vbox);

	window->add (*hbox);
	window->show_all ();
}

void *
ARDOUR_UI::_start_engine (void *arg)
{
	ARDOUR_UI *ui = reinterpret_cast<ARDOUR_UI*> (arg);

	ui->engine->start();
	Gtkmmext::UI::instance()->call_slot (slot (ui, &ARDOUR_UI::hide_wait_for_loading));

	Gtkmmext::UI::instance()->call_slot (slot (ui, &ARDOUR_UI::notify_engine_started));

	return 0;
}

gint
ARDOUR_UI::start_engine ()
{
	_start_engine (this);
#if 0
	pthread_t thread_id;
	pthread_create (&thread_id, 0, _start_engine, this);
	pthread_detach (thread_id);
#endif
	return FALSE;
}

void
ARDOUR_UI::notify_engine_started ()
{
	/* XXX: this is necessary for reasons not entirely clear.
	   without reshowing the guis after the engine has started
	   often puts GTK into a strange unhappy state
	*/
	show();
}

void
ARDOUR_UI::show_wait_for_loading ()
{
	wait_for_loading_label = manage (new Gtk::Label);
	wait_for_loading_label->set_text (_("Please Wait\nArdour is loading and configuring your session\n"));

	wait_for_loading_hbox = manage (new Gtk::HBox);
	wait_for_loading_hbox->pack_start (*wait_for_loading_label);
	wait_for_loading_hbox->set_border_width (10);
	
	wait_for_loading_window = new ArdourDialog;
	wait_for_loading_window->set_position (GTK_WIN_POS_CENTER);
	editor->ensure_float (*wait_for_loading_window);

	wait_for_loading_window->realize();
	wait_for_loading_window->get_window().set_decorations (GdkWMDecoration (GDK_DECOR_BORDER|GDK_DECOR_RESIZEH));
	wait_for_loading_window->get_window().raise ();

	wait_for_loading_window->add (*wait_for_loading_hbox);
	wait_for_loading_window->show_all ();

	/* make it really show */

	flush_pending();
}

void
ARDOUR_UI::hide_wait_for_loading ()
{
	if (wait_for_loading_window) {
		wait_for_loading_window->hide_all();
		delete wait_for_loading_window;
		wait_for_loading_window = 0;
	}
}

void
ARDOUR_UI::update_clocks ()
{
	EMIT_SIGNAL Clock (session->transport_frame());
}

void
ARDOUR_UI::start_clocking ()
{
	clock_signal_connection = RapidScreenUpdate.connect (slot (*this, &ARDOUR_UI::update_clocks));
}

void
ARDOUR_UI::stop_clocking ()
{
	clock_signal_connection.disconnect ();
}
	
void
ARDOUR_UI::toggle_clocking ()
{
#if 0
	if (clock_button.get_active()) {
		start_clocking ();
	} else {
		stop_clocking ();
	}
#endif
}

gint
ARDOUR_UI::_blink (void *arg)

{
	((ARDOUR_UI *) arg)->blink ();
	return TRUE;
}

void
ARDOUR_UI::blink ()
{
	EMIT_SIGNAL Blink (blink_on = !blink_on);
}

void
ARDOUR_UI::start_blinking ()
{
	/* Start the blink signal. Everybody with a blinking widget
	   uses Blink to drive the widget's state.
	*/

	if (blink_timeout_tag < 0) {
		blink_on = false;	
		blink_timeout_tag = gtk_timeout_add (240, _blink, this);
	}
}

void
ARDOUR_UI::stop_blinking ()
{
	if (blink_timeout_tag >= 0) {
		gtk_timeout_remove (blink_timeout_tag);
		blink_timeout_tag = -1;
	}
}


void
ARDOUR_UI::add_diskstream_to_menu (DiskStream& dstream)
{
	using namespace Gtk;
	using namespace Menu_Helpers;

	if (dstream.hidden()) {
		return;
	}

	MenuList& items = diskstream_menu->items();
	items.push_back (MenuElem (dstream.name(), bind (slot (this, &ARDOUR_UI::diskstream_selected), (gint32) dstream.id())));
}
	
void
ARDOUR_UI::diskstream_selected (gint32 id)
{
	selected_dstream = id;
	Main::quit ();
}

gint32
ARDOUR_UI::select_diskstream (GdkEventButton *ev)

{
	using namespace Gtk;
	using namespace Menu_Helpers;

	if (session == 0) {
		return -1;
	}

	diskstream_menu = new Menu();
	using namespace Gtk;
	using namespace Menu_Helpers;

	MenuList& items = diskstream_menu->items();
	items.push_back (MenuElem (_("No Stream"), (bind (slot (this, &ARDOUR_UI::diskstream_selected), -1))));

	session->foreach_diskstream (this, &ARDOUR_UI::add_diskstream_to_menu);

	if (ev) {
		diskstream_menu->popup (ev->button, ev->time);
	} else {
		diskstream_menu->popup (0, 0);
	}

	selected_dstream = -1;

	Main::run ();

	delete diskstream_menu;

	return selected_dstream;
}

void
ARDOUR_UI::name_io_setup (AudioEngine& engine, 
			  string& buf,
			  IO& io,
			  bool in)
{
	if (in) {
		if (io.n_inputs() == 0) {
			buf = _("none");
			return;
		}
		
		/* XXX we're not handling multiple ports yet. */

		const char **connections = io.input(0)->get_connections();
		
		if (connections == 0 || connections[0] == '\0') {
			buf = _("off");
		} else {
			buf = connections[0];
		}

		free (connections);

	} else {

		if (io.n_outputs() == 0) {
			buf = _("none");
			return;
		}
		
		/* XXX we're not handling multiple ports yet. */

		const char **connections = io.output(0)->get_connections();
		
		if (connections == 0 || connections[0] == '\0') {
			buf = _("off");
		} else {
			buf = connections[0];
		}

		free (connections);
	}
}

void
ARDOUR_UI::snapshot_session ()
{
	ArdourPrompter prompter (true);
	string now;
	time_t n;

	time (&n);
	now = ctime (&n);
	now = now.substr (0, now.length() - 1);

	prompter.set_prompt (_("Name for snapshot"));
	prompter.set_initial_text (now);
	prompter.done.connect (Gtk::Main::quit.slot());
	prompter.show_all ();

	Gtk::Main::run ();

	if (prompter.status == Gtkmmext::Prompter::entered) {
		string snapname;
		
		prompter.get_result (snapname);
		if (snapname.length()){
			save_state (snapname);
		}
	}
}
		
void
ARDOUR_UI::save_state (string name)
{
	if (session) {
		if (name.length() == 0) {
			name = "default";
		}
		session->save_state (name);
	}
}

void
ARDOUR_UI::restore_state (string name)
{
	if (session) {
		if (name.length() == 0) {
			name = "default";
		}
		session->restore_state (name);
	}
}

void
ARDOUR_UI::allow_focus (bool yn)
{
	if (keyboard) {
		keyboard->allow_focus (yn);
	}
}

void
ARDOUR_UI::primary_clock_value_changed ()
{
	if (session) {
		session->request_locate (primary_clock.current_time ());
	}
}

void
ARDOUR_UI::secondary_clock_value_changed ()
{
	if (session) {
		session->request_locate (secondary_clock.current_time ());
	}
}

void
ARDOUR_UI::rec_enable_button_blink (bool onoff, DiskStream *dstream, Widget *w)
{
	if (session && dstream && dstream->record_enabled()) {

		Session::RecordState rs;
		
		rs = session->record_status ();

		switch (rs) {
		case Session::Disabled:
		case Session::Enabled:
			if (w->get_state() != GTK_STATE_SELECTED) {
				w->set_state (GTK_STATE_SELECTED);
			}
			break;

		case Session::Recording:
			if (w->get_state() != GTK_STATE_ACTIVE) {
				w->set_state (GTK_STATE_ACTIVE);
			}
			break;
		}

	} else {
		if (w->get_state() != GTK_STATE_NORMAL) {
			w->set_state (GTK_STATE_NORMAL);
		}
	}
}

void
ARDOUR_UI::transport_rec_enable_blink (bool onoff) 
{
	if (session == 0) {
		return;
	}
	
	switch (session->record_status()) {
	case Session::Enabled:
		if (onoff) {
			rec_button.set_state (GTK_STATE_ACTIVE);
		} else {
			rec_button.set_state (GTK_STATE_NORMAL);
		}
		break;

	case Session::Recording:
		rec_button.set_state (GTK_STATE_ACTIVE);
		break;

	default:
		rec_button.set_active (false);
		rec_button.set_state (GTK_STATE_NORMAL);
		break;
	}
}

gint
ARDOUR_UI::generic_focus_in_event (GdkEventFocus *ev)
{
	ARDOUR_UI::instance()->allow_focus (true);
	return FALSE;
}

gint
ARDOUR_UI::generic_focus_out_event (GdkEventFocus *ev)
{
	ARDOUR_UI::instance()->allow_focus (false);
	return FALSE;
}

gint
ARDOUR_UI::hide_and_quit (GdkEventAny *ev, ArdourDialog *window)
{
	window->hide();
	Gtk::Main::quit ();
	return TRUE;
}

void
ARDOUR_UI::start_keyboard_prefix ()
{
	keyboard->start_prefix();
}

void
ARDOUR_UI::save_template ()

{
	ArdourPrompter prompter (true);
	prompter.set_prompt (_("Name for mix template:"));
	prompter.set_initial_text(session->name() + _("-template"));

	prompter.done.connect(Gtk::Main::quit.slot());
	prompter.show_all();
	
	Gtk::Main::run();
	
	if (prompter.status == Gtkmmext::Prompter::entered) {
		string name;

		prompter.get_result (name);

		if (name.length()) {
			session->save_template (name);
		}
	}
}

void
ARDOUR_UI::_mix_template_refiller (Gtk::CList &clist, void *arg)

{
	((ARDOUR_UI *) arg)->mix_template_refiller (clist);
}

void
ARDOUR_UI::mix_template_refiller (Gtk::CList &clist)

{
	const gchar *rowdata[2];
	list<string> templates;
	list<string>::iterator i;
	
	Session::get_template_list(templates);
	
	rowdata[0] = _("blank");
	clist.insert_row (0, rowdata);

	guint row;
	for (row=1, i=templates.begin(); i != templates.end(); ++row, ++i) {
		rowdata[0] = (*i).c_str();
		clist.insert_row (row, rowdata);
	}
}

void
ARDOUR_UI::build_new_session_window ()
{
	new_session_window = new ArdourDialog;

	Gtk::VBox *vpacker = manage (new Gtk::VBox);
	Gtk::HBox *button_packer = manage (new Gtk::HBox);
	Gtk::Button *cancel_button = manage (new Gtk::Button (_("cancel")));
	Gtk::Button *ok_button = manage (new Gtk::Button (_("ok")));
	Gtk::VBox *entry_packer = manage (new Gtk::VBox);
	Gtk::Label *entry_label = manage (new Gtk::Label (_("Session name")));

	new_session_entry = manage (new Gtk::Entry());

	button_packer->pack_start (*ok_button);
	button_packer->pack_start (*cancel_button);

	entry_label->set_name ("NewSessionLabel");

	entry_packer->set_spacing (10);
	entry_packer->pack_start (*entry_label, false, false);
	entry_packer->pack_start (*new_session_entry, false, false);

	vpacker->set_border_width (10);
	vpacker->set_spacing (5);
	vpacker->pack_start (*entry_packer, false, false);
	vpacker->pack_start (template_selector);
	vpacker->pack_start (*button_packer, false, false);

	new_session_window->add (*vpacker);
	new_session_window->set_name ("NewSessionWindow");
	new_session_window->set_usize (200, 200);
	new_session_window->set_title (_("ardour: new session"));
	new_session_window->realize();
	new_session_window->get_window().set_decorations (GdkWMDecoration(GDK_DECOR_BORDER|GDK_DECOR_TITLE|GDK_DECOR_RESIZEH));

	new_session_window->delete_event.connect (bind (slot (Gtkmmext::UI::just_hide_it), static_cast<Gtk::Window*>(new_session_window)));
	cancel_button->clicked.connect (bind (slot (*this, &ARDOUR_UI::hide_dialog), new_session_window));
	ok_button->clicked.connect (slot (*this, &ARDOUR_UI::new_session_ok_clicked));
	new_session_entry->activate.connect (slot (*this, &ARDOUR_UI::new_session_ok_clicked));
	template_selector.choice_made.connect(slot(this, &ARDOUR_UI::new_session_template_choice));
	template_selector.shift_made.connect (slot (this, &ARDOUR_UI::mix_template_shift));
	template_selector.control_made.connect (slot (this, &ARDOUR_UI::mix_template_control));
}

void 
ARDOUR_UI::new_session_ok_clicked ()
{
	string session_name = new_session_entry->get_text();

	if (session_name.length() != 0) {
		if (template_name == _("blank")){
			load_session (session_name, "default");
		} else {
			load_session (session_name, "default", &template_name);
		}
	}
	allow_focus(false);
	new_session_window->hide();
}

void
ARDOUR_UI::new_session_template_choice (Gtkmmext::Selector* selector, Gtkmmext::SelectionResult* res)
{
	if (res && res->text){
		template_name = *res->text;
	} else {
		template_name = _("blank");
	}

}

void
ARDOUR_UI::new_session ()
{
	if (new_session_window == 0){
		build_new_session_window();
	}
	template_selector.rescan();
	template_selector.clist().row(0).select();
	template_name = _("blank");
	new_session_entry->set_text("");
	new_session_window->show_all();
	allow_focus(true);
}

void
ARDOUR_UI::mix_template_shift (Gtkmmext::Selector* selector, Gtkmmext::SelectionResult* res)
{
	if (res && res->text){
		Session::delete_template(*res->text);
		template_selector.rescan();
	}
}

void
ARDOUR_UI::mix_template_control (Gtkmmext::Selector* selector, Gtkmmext::SelectionResult* res)
{
	if (res && res->text) {
		ArdourPrompter prompter (true);
		prompter.set_prompt(_("Name for mix template:"));

		string old_name = *(res->text);
		prompter.set_initial_text (old_name);
		prompter.done.connect (Gtk::Main::quit.slot());
		prompter.show_all();
		
		Gtk::Main::run();
		
		if (prompter.status == Gtkmmext::Prompter::entered) {
			string name;

			prompter.get_result (name);

			if (name.length() && name != old_name) {
				Session::rename_template(old_name, name);
				template_selector.rescan();
			}
		}
	}
}

void
ARDOUR_UI::hide_dialog (ArdourDialog *dialog)
{
	dialog->hide_all();
}

jack_nframes_t
ARDOUR_UI::get_nudge_distance (jack_nframes_t pos, jack_nframes_t& next)
{
	jack_nframes_t ret;

	ret = nudge_clock.current_duration (pos);
	next = ret + 1; /* XXXX fix me */

	return ret;
}

void
ARDOUR_UI::show ()
{
	if (editor) {
		editor->show_window ();
	}

	if (session && mixer) {
		mixer->show_window ();
	}
	
	if (about) {
		about->get_window().raise ();
	}
}

void
ARDOUR_UI::show_splash ()
{
	if (about == 0) {
		about = new About;
		about->show_all();
		about->get_window().raise ();
	}
	else {
		about->show_all ();
		about->get_window().raise ();
	}
}
