/*
    Copyright (C) 2000 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: audio_time_axis.h,v 1.5 2003/06/11 05:31:17 pbd Exp $
*/

#ifndef __ardour_trackview_h__
#define __ardour_trackview_h__

#include <gtk--.h>
#include <gtk-canvas.h>
#include <gtkmmext/selector.h>
#include <gtkmmext/popup_selector.h>
#include <ardour/playlist.h>

#include "ardour_dialog.h"
#include "route_ui.h"
#include "enums.h"
#include "time_axis_view.h"

namespace ALSA {
	class MultiChannelDevice;
}

namespace ARDOUR {
	class Session;
	class DiskStream;
	class RouteGroup;
	class Redirect;
	class Insert;
	class Location;
}

namespace LADSPA {
	class Manager;
	class Plugin;
}

class Editor;
class AudioThing;
class StreamView;
class RegionView;
class AutomationGainLine;
class AutomationPanLine;
class RedirectAutomationLine;
class TimeSelection;

class AudioTimeAxisView : public RouteUI, public TimeAxisView
{
  public:
 	AudioTimeAxisView (Editor&, ARDOUR::Session&, ARDOUR::Route&, Gtk::Widget *canvas);
 	virtual ~AudioTimeAxisView ();
  
	void automation_control_point_changed (ARDOUR::AutomationType);
	void add_gain_automation_point_event (GtkCanvasItem *item, GdkEvent *event);

	void set_samples_per_unit (double);
 	void set_height (TimeAxisView::TrackHeight);
  	void order_selection_trims (GtkCanvasItem *item, bool put_start_on_top);
	void set_show_waveforms (bool yn);
	void show_selection (TimeSelection&);
	void reshow_selection (TimeSelection&);
	void hide_selection ();
	void show_timestretch (TimeSelection&);
	void hide_timestretch ();
	void set_selected_regionviews (RegionSelection&);

	string name() const;

	ARDOUR::RouteGroup* edit_group() const;
	ARDOUR::Playlist* playlist() const;

	/* need accessors/mutators */

	StreamView      *view;

  private:
	friend class StreamView;
	friend class RegionView;

  	bool                history_mapped;
  
  	void set_height_from_route ();
  
  	GtkCanvasItem      *selection_group;

	AutomationGainLine *gain_line;
	AutomationPanLine  *pan_line;

	void update_automation_view (ARDOUR::AutomationType);
	void reset_redirect_automation_curves ();

	Gtk::HBox  other_button_hbox;

	Gtk::Table button_table;

	Gtk::Button       redirect_button;
	Gtk::Button       edit_group_button;
	Gtk::Button       history_button;
	Gtk::Button       playlist_button;
	Gtk::ToggleButton loop_button;
	Gtk::Button       size_button;
	Gtk::Button       automation_button;
	Gtk::Button       hide_button;
	Gtk::Button       visual_button;

	/* History handling */
	
	void history_click ();
	
	Gtkmmext::Selector   history_selector;
	ArdourDialog          history_display;

	void really_refill_history_list (const ARDOUR::Playlist::StateMap& states, state_id_t current);
	static void _refill_history_list (Gtk::CList&, void *);
	void refill_history_list (Gtk::CList&);
	void random_history_selection (Gtkmmext::Selector *, Gtkmmext::SelectionResult *);
	void history_map_follower (bool);

	void diskstream_changed (void *src);
	void update_diskstream_display ();
	
	gint edit_click  (GdkEventButton *);

	/* Redirect handling */

	ArdourDialog redirect_window;
	Gtk::CList  redirect_display;
	Gtk::Button redirect_add_button;
	Gtk::Button redirect_remove_button;
	Gtk::Button redirect_up_button;
	Gtk::Button redirect_down_button;
	Gtk::Button redirect_edit_button;
	Gtk::HBox   redirect_button_box;
	Gtk::VBox   redirect_vpacker;
	Gtk::HBox   redirect_display_hpacker;

	int redirect_selected_row;

	void bind_new_redirect_plugin (LADSPA::Plugin *);

	void build_redirect_window ();
	void redirect_click ();
	void redirect_add ();
	void redirect_remove ();
	void redirect_edit ();
	void redirect_relist ();
	void redirect_row_selected (gint row, gint col, GdkEvent *ev);
	void add_to_redirect_display (ARDOUR::Redirect *);
	void redirects_changed (void *);
	void redirect_automation_changed (ARDOUR::Redirect*, unsigned long port);

	/* track loop GUI */

	Gtkmmext::PopupSelector *loop_selector;

	static void loop_selector_refill (Gtk::CList &clist, void *arg);
	void loop_selector_refill_internal (ARDOUR::Locations::LocationList&, Gtk::CList &);
	void loop_selection_made (Gtkmmext::PopupSelector *, Gtkmmext::SelectionResult *);
	void end_loop_selection ();
	void loop_toggle ();
	void loop_set (ARDOUR::Location *);
	void start_loop_selection ();
	bool ignore_loop_toggle;
	
	SigC::Connection loop_set_connection;
	SigC::Connection modified_connection;
	SigC::Connection state_changed_connection;
	SigC::Connection state_pushed_connection;

	void take_name_changed (void *);
	void route_name_changed (void *);
	void name_entry_activated ();
	void name_entry_changed ();
	gint name_entry_focus_out_handler (GdkEventFocus*);
	gint name_entry_key_release_handler (GdkEventKey*);
	gint name_entry_button_release_handler (GdkEventButton*);
	gint name_entry_button_press_handler (GdkEventButton*);
	void on_area_realize ();

	virtual void label_view ();

	Gtk::Menu edit_group_menu;
	Gtk::RadioMenuItem::Group edit_group_menu_radio_group;

	void add_edit_group_menu_item (ARDOUR::RouteGroup *);
	void set_edit_group_from_menu (ARDOUR::RouteGroup *);
	
	void reset_samples_per_unit ();

	void select_track_color();
	
	virtual void build_display_menu ();

	Gtk::CheckMenuItem *waveform_item;
	Gtk::RadioMenuItem *traditional_item;
	Gtk::RadioMenuItem *rectified_item;

	void toggle_show_waveforms ();

	void set_waveform_shape (WaveformShape);
	void toggle_waveforms ();

	Gtk::Menu *playlist_menu;
	Gtk::Menu *playlist_action_menu;
	Gtk::MenuItem *choose_playlist_item;
	Gtk::MenuItem *playlist_item;

	/* playlist */

	void set_playlist (ARDOUR::Playlist *);
	void playlist_click ();

	void playlist_changed ();
	void playlist_state_changed ();
	void playlist_state_pushed ();

	void choose_playlist ();
	void add_playlist_to_playlist_menu (ARDOUR::Playlist*);
	void playlist_selected (ARDOUR::Playlist*);
	void use_new_playlist ();
	void use_copy_playlist ();
	void clear_playlist ();

	/* automation stuff */
	
	Gtk::Menu* automation_action_menu;
	Gtk::Menu* automation_clear_menu;
	Gtk::Menu* automation_hide_menu;
	Gtk::Menu* automation_show_menu;

	gint handle_automation_menu_map_event (GdkEventAny*, Gtk::Menu*);

	void automation_click ();
	void clear_automation ();
	void hide_all_automation ();
	void show_all_automation ();
	
	gint handle_display_menu_map_event (GdkEventAny *);
	gint handle_playlist_menu_map_event (GdkEventAny *);

	Gtk::Menu subplugin_menu;
	gint build_subplugin_menu (GdkEventAny *);
	void add_redirect_to_subplugin_menu (ARDOUR::Redirect *);

	vector<RedirectAutomationLine*> redirect_automation_curves;
	RedirectAutomationLine *find_redirect_automation_curve (ARDOUR::Redirect*,unsigned long);
	void add_redirect_automation_curve (ARDOUR::Redirect*, unsigned long);
	void add_redirect_automation_curves (ARDOUR::Redirect*);
	void setup_redirect_automation ();
	void remove_redirect_automation_curve (ARDOUR::Redirect*, RedirectAutomationLine*);

	list<SelectionRect*> free_selection_rects;
	list<SelectionRect*> used_selection_rects;

	SelectionRect* get_selection_rect(unsigned long id);

	GtkCanvasItem *timestretch_rect;

	void timestretch (jack_nframes_t start, jack_nframes_t end);

	void visual_click ();
	void hide_click ();
	gint when_displayed (GdkEventAny*);
};

#endif /* __ardour_trackview_h__ */

