/*
    Copyright (C) 2003 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: axis_view.h,v 1.1 2003/02/21 15:32:28 pbd Exp $
*/

#ifndef __ardour_gtk_axis_view_h__
#define __ardour_gtk_axis_view_h__

#include <list>
#include <gtk--.h>
#include <pbd/xml++.h>
#include "prompter.h"

namespace ARDOUR {
	class Session;
}

class TimeSelection;
class RegionView;

/**
 * AxisView defines the abstract base class for time-axis trackviews and routes.
 *
 */
class AxisView : public SigC::Object
{
  public:
	/**
	 * Returns the current 'Track' Color
	 *
	 * @return the current Track Color
	 */
	Gdk_Color color() const { return _color; }

	ARDOUR::Session& session() const { return _session; }

	virtual string name() const = 0;

	SigC::Signal0<void> GoingAway;

  protected:

	AxisView (ARDOUR::Session& sess);
	virtual ~AxisView();
	
	// 'Track' Colors

	/**
	 * Display a color selector to allow a new TrackView color to be chosen
	 *
	 * @return true if a new color was chosen; false otherwise
	 */
	virtual bool choose_color ();

	/**
	 * Sst the color of this ViewUI
	 *
	 * @param c the new color
	 */
	virtual void set_color (Gdk_Color c);

	/**
	 * Generate a new random TrackView color, unique from those colors already used.
	 *
	 * @return the unique random color.
	 */
	static GdkColor unique_random_color();

	/**
	 * Ensure that we have a valid XML Node for saving state information.
	 *
	 */
	virtual void ensure_xml_node() = 0;

	/**
	 * Handle the removing of this Route
	 *
	 * @todo we should rename this to something more abstract
	 */
	virtual void remove_this_route() = 0;

	/**
	 * Handle when our (route) data object is removed.
	 *
	 */
	virtual void route_removed() = 0;

	/**
	 * Display the name dialog to ask for a new (route) name.
	 * 
	 * @todo this should be renamed
	 */
	virtual void start_route_rename();

	/**
	 * Closes the new name dialog and sets the name upon the route label
 	 *
	 */
	virtual void finish_route_rename();

	/**
	 * Handle when out (route) data objects has had its name changed.
	 *
	 */
	virtual void name_changed (void *src) = 0;

	/**
	 * Display/Get the name for the view name label.
	 * Subclasses should override this to do something useful
	 */

	virtual void label_view ();

	/**
	 * Reset the number of samples per canvas unit ("zoom")
	 */

	ARDOUR::Session& _session;
	Gdk_Color _color;

	static list<GdkColor> used_colors;

	XMLNode *xml_node;

	Gtk::Label name_label;

	ArdourPrompter  name_prompter;

}; /* class AxisView */

#endif /* __ardour_gtk_axis_view_h__ */

