/*
    Copyright (C) 1999-2002 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __ardour_library_ui_h__
#define __ardour_library_ui_h__

#include <map>
#include <string>
#include <sys/stat.h>
#include <sys/types.h>

#include <sigc++/signal_system.h>
#include <gtk--.h>
#include <gtkmmext/selector.h>

#include "ardour_dialog.h"

using std::string;
using std::map;

class LibraryTree : public ArdourDialog
{
  public:
	LibraryTree (string action, bool import, bool multi, bool persist);
	~LibraryTree ();

	SigC::Signal2<void, string, bool> file_chosen;
  
  private:
	const bool importable;
	const bool multiable;
	const bool persistant;

	map<string, Gtk::TreeItem*> uri_mapping;
	map<string, string> uri_parent; // this ugly, but necessary

	string member_selected;
	string group_selected;
	
	Gtk::VBox main_box;
	Gtk::VBox framed_box;
	Gtk::HBox btn_box_top;
	Gtk::HBox btn_box_bottom;
	Gtk::HBox import_box;

	Gtk::ScrolledWindow scroll;
	Gtk::Tree tree;

	Gtk::Button import_btn;
	Gtk::Button add_btn;
	Gtk::Button remove_btn;
	Gtk::Button info_btn;
	Gtk::Button find_btn;
	Gtk::Button folder_btn;
	Gtk::CheckButton multichan_check;

	Gtk::FileSelection files_select;

	void file_ok_clicked ();
	void file_cancel_clicked ();
	
	static bool busy;

	void add_btn_clicked ();
	void folder_btn_clicked ();
	void remove_btn_clicked ();
	void find_btn_clicked ();
	void info_btn_clicked ();
	void import_btn_clicked ();

	void file_found (string uri, bool multi);

	void cb_group_select (Gtk::TreeItem* item, string uri);
	void cb_member_select (Gtk::TreeItem* item, string uri);
	gint member_button_press (GdkEventButton* ev);

	void populate ();
	void subpopulate (Gtk::Tree*, string group);

	void added_group (string, string);
	void removed_group (string);
	void added_member (string, string);
	void removed_member (string);
	
	void cancel_import_clicked ();
};

class SoundFileBox : public ArdourDialog 
{
  public:
	SoundFileBox (string uri);
	~SoundFileBox ();

	SigC::Signal2<void, string, bool> file_chosen;

  private:
	string uri;

	char* playcmd;
	pid_t current_pid;
	
	Gtk::Label label;
	Gtk::Label path;
	Gtk::Label length;
	Gtk::Label format;
	Gtk::Label channels;
	Gtk::Label samplerate;

	Gtkmmext::Selector fields;
	string selected_field;
	
	Gtk::VBox main_box;
	Gtk::HBox top_box;
	Gtk::HBox bottom_box;

	Gtk::Button play_btn;
	Gtk::Button add_field_btn;
	Gtk::Button remove_field_btn;

	static void _fields_refiller (Gtk::CList &list, void* arg);
	void fields_refiller (Gtk::CList &clist);
	void setup_labels (string uri);
	void setup_fields ();

	void play_btn_clicked ();
	void add_field_clicked ();
	void remove_field_clicked ();

	void field_selected (Gtkmmext::Selector *selector, 
						 Gtkmmext::SelectionResult *re);
	void field_chosen (Gtkmmext::Selector *selector, 
						 Gtkmmext::SelectionResult *re);
	void stop_play ();
};

class SearchSounds : public ArdourDialog 
{
  public:
    SearchSounds (bool import, bool multi);
	~SearchSounds ();

	SigC::Signal2<void, string, bool> file_chosen;

  private:
	const bool importable;
	const bool multiable;
	
	Gtk::Button find_btn;

	Gtk::RadioButton and_rbtn;
	Gtk::RadioButton or_rbtn;

	Gtkmmext::Selector fields;
	string selected_field;

	Gtk::VBox main_box;
	Gtk::HBox rbtn_box;
	Gtk::HBox bottom_box;

	static void _fields_refiller (Gtk::CList &list, void* arg);
	void fields_refiller (Gtk::CList &clist);
	void setup_fields ();

	void field_selected (Gtkmmext::Selector *selector, 
						 Gtkmmext::SelectionResult *re);

	void find_btn_clicked ();

	void file_found (string uri, bool multi);
};

class SearchResults : public ArdourDialog
{
  public:
	SearchResults (map<string,string> field_values, bool and_search, 
				   bool import, bool multi);
	~SearchResults ();

	SigC::Signal2<void, string, bool> file_chosen;

  private:
	const bool importable;
	const bool multiable;
	
	map<string,string> search_info;
	bool search_and;
	string selection;

	Gtk::VBox main_box;
	Gtk::HBox button_box;
	Gtk::HBox import_box;
	
	Gtk::Button info_btn;
	Gtk::Button import_btn;
	Gtk::CheckButton multichan_check;

	Gtkmmext::Selector results;
	static void _results_refiller (Gtk::CList &list, void* arg);
	void results_refiller (Gtk::CList &clist);

	void info_clicked ();
	void import_clicked ();

	void result_chosen (Gtkmmext::Selector *selector, 
						 Gtkmmext::SelectionResult *re);
	void result_selected (Gtkmmext::Selector *selector, 
						 Gtkmmext::SelectionResult *re);
};

#endif // __ardour_library_ui_h__
