/*
    Copyright (C) 1999-2002 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: location_ui.h,v 1.3 2002/11/04 17:43:04 pbd Exp $
*/

#ifndef __ardour_location_ui_h__
#define __ardour_location_ui_h__

#include <gtk--.h>
#include <ardour/location.h>
#include <ardour/session.h>

#include "ardour_dialog.h"

namespace ARDOUR {
	class LocationStack;
	class Session;
}

class LocationUI : public ArdourDialog
{
  public:
	LocationUI ();
	~LocationUI ();

	void set_session (ARDOUR::Session *);

  private:
	ARDOUR::LocationStack          *locations;

	void session_gone();

	Gtk::Adjustment start_adjustment;
	Gtkmmext::AutoSpin start_spinner;

	Gtk::Adjustment end_adjustment;
	Gtkmmext::AutoSpin end_spinner;
	
	Gtkmmext::PixmapButton   *start_grab_session;
	Gtkmmext::PixmapButton   *start_forward;
	Gtkmmext::PixmapButton   *start_backward;
	Gtk::Label start_forward_label;
	Gtk::Label start_grab_label;
	Gtk::Label start_backward_label;
	Gtk::VBox start_forward_box;
	Gtk::VBox start_grab_box;
	Gtk::VBox start_backward_box;
	Gtk::HBox start_sub_hbox;

	Gtkmmext::PixmapButton   *end_grab_session;
	Gtkmmext::PixmapButton   *end_forward;
	Gtkmmext::PixmapButton   *end_backward;
	Gtk::Label end_forward_label;
	Gtk::Label end_grab_label;
	Gtk::Label end_backward_label;
	Gtk::VBox end_forward_box;
	Gtk::VBox end_grab_box;
	Gtk::VBox end_backward_box;
	Gtk::HBox end_sub_hbox;

	Gtk::Table start_end_table;

	Gtk::Label next_label;
	Gtk::Label previous_label;
	Gtk::HBox next_box;
	Gtk::HBox previous_box;
	Gtk::HBox next_prev_hbox;

	Gtk::VBox  location_vpacker;
	Gtk::HBox  location_hpacker;

	Gtk::CList          location_clist;
	Gtk::ScrolledWindow location_scroller;
	
	Gtk::Label start_label;
	Gtk::Frame start_frame;
	Gtk::VBox  start_vbox;

	Gtk::Label end_label;
	Gtk::Frame end_frame;
	Gtk::VBox  end_vbox;

	Gtk::Pixmap             *start_pixmap;
	Gtk::EventBox            start_button;
	Gtk::VBox                start_button_vbox;

	Gtk::Pixmap             *end_pixmap;
	Gtk::EventBox            end_button;
	Gtk::VBox                end_button_vbox;

	Gtk::Pixmap             *set_next_pixmap;
	Gtk::EventBox            set_next_button;

	SigC::Connection start_changed_connection;
	SigC::Connection end_changed_connection;

	static GdkPixmap *check_pixmap;
	static GdkBitmap *check_mask;
	static GdkPixmap *empty_pixmap;
	static GdkBitmap *empty_mask;

	static const int location_columns = 8;

	void build_location_window ();
	
	gint              find_row_from_location (ARDOUR::Location *);
	ARDOUR::Location *find_location_from_row (gint);

	/* When the session changes it auto{loop,punch} point,
	   it sends a signal that is caught by one of
	   these functions.
	*/

	void auto_punch_changed (ARDOUR::Location *);
	void auto_loop_changed (ARDOUR::Location *);

	void auto_punch_toggle ();
	void auto_loop_toggle ();

	/* When any location changes it start
	   or end points, it sends a signal that is caught
	   by one of these functions
	*/

	void location_end_changed (ARDOUR::Location *);
	void location_start_changed (ARDOUR::Location *);
	void location_name_changed (ARDOUR::Location *);
	void current_location_changed (ARDOUR::Location *);
	void location_changed (ARDOUR::Location *);
	void location_flags_changed (ARDOUR::Location *, void *src);

	guint32 i_am_the_modifier;

	gint location_fake_button_press (GdkEventButton *, int);

	void select_new_location_name (ARDOUR::Location *);
	
	void refresh_location_list ();
	void add_new_location (ARDOUR::Location *);
	void map_location_change (ARDOUR::Location *, bool update_start, bool update_end);
	void map_locations (ARDOUR::Locations::LocationList&);
	void map_location (guint32 row, ARDOUR::Location *);
	void map_start (guint32 row, ARDOUR::Location *, bool with_length);
	void map_end (guint32 row, ARDOUR::Location *, bool with_length);
	void map_length (guint32 row, ARDOUR::Location *);
	void map_name (guint32 row, ARDOUR::Location *);

	void set_location_name ();
	void set_start ();
	void set_end ();
	void set_start_from_session ();
	void set_end_from_session   ();
	gint set_next_from_session (GdkEventButton *);

	void location_show_check (gint row, gint col, bool yn);

	gint location_list_button_press_event (GdkEventButton *ev);

	void row_selected (gint row, gint col, GdkEvent *);
	void col_selected (gint col);
};

#endif // __ardour_location_ui_h__
