/*
    Copyright (C) 2000 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: redirect_automation_line.h,v 1.7 2003/03/31 03:38:57 pbd Exp $
*/

#ifndef __ardour_gtk_redirect_automation_line_h__
#define __ardour_gtk_redirect_automation_line_h__

#include <ardour/ardour.h>
#include <gtk-canvas.h>
#include <gtk--.h>

#include "automation_line.h"

namespace ARDOUR {
	class Session;
	class Redirect;
}

class Editor;
class AudioTimeAxisView;

class RedirectAutomationLine : public AutomationLineList
{
  public:
	RedirectAutomationLine (string name, ARDOUR::Redirect&, unsigned long port, ARDOUR::Session&, Editor&, GtkCanvasItem* parent,
				ARDOUR::AutomationList&, 
				gint (*point_callback)(GtkCanvasItem*, GdkEvent*, gpointer),
				gint (*line_callback)(GtkCanvasItem*, GdkEvent*, gpointer));
	
	unsigned long port() const { return _port; }
	ARDOUR::Redirect& redirect() const { return _redirect; }

	void reset();
	string get_verbose_cursor_string (float);

  private:
	ARDOUR::Session& session;
	ARDOUR::Redirect& _redirect;
	unsigned long _port;
	float upper;
	float lower;
	float range;

	void translate_yaxis_fraction (double&);
	void change_model (unsigned long, double x, double y);
	void change_model_range (unsigned long, unsigned long, double delta);
	void remove_model_range (unsigned long start, unsigned long end);
	void add_to_model (double, double);
	UndoAction get_memento();
};


#endif /* __ardour_gtk_region_gain_line_h__ */
