/*
    Copyright (C) 2001 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: regionview.h,v 1.45 2003/05/09 18:04:35 pbd Exp $
*/

#ifndef __gtk_ardour_region_view_h__
#define __gtk_ardour_region_view_h__

#include <vector>
#include <gtk--.h>
#include <gtk-canvas.h>
#include <sigc++/signal_system.h>
#include <ardour/region.h>

#include "automation_line.h"
#include "enums.h"

class AudioTimeAxisView;
class RegionEditor;
class RegionView;
class RegionGainLine;

struct RegionView : public SigC::Object
{
    RegionView (GtkCanvasGroup *, 
		AudioTimeAxisView&,
		ARDOUR::Region&,
		double initial_samples_per_unit,
		double amplitude_above_axis,
		GdkColor& base_color);
    ~RegionView ();
    
    void set_height (gdouble);
    void set_frame_position (jack_nframes_t pos);
    void set_samples_per_unit (gdouble spp);
    void set_amplitude_above_axis (gdouble spp);

    void set_color (GdkColor&);

    void raise ();
    void raise_to_top ();
    void lower ();
    void lower_to_bottom ();
    
    void set_envelope_visible (bool);
    void set_waveform_visible (bool yn);

    
    GtkCanvasItem *group;/* the group */
    GtkCanvasItem *frame; /* simplerect */
    GtkCanvasItem *name; /* text */

    vector<GtkCanvasItem *> waves; /* waveviews */
    GtkCanvasItem *sync_mark; /* polgyon for sync position */
    GtkCanvasItem *name_highlight; /* simplerect */
    GtkCanvasItem *no_wave_msg; /* text */
    GtkCanvasItem *zero_line; /* simpleline */

    RegionGainLine* gain_line;

    AudioTimeAxisView& trackview;
    ARDOUR::Region& region;

    bool valid; /* see StreamView::redisplay_diskstream() */

    void region_moved (void *);
    void region_resized (ARDOUR::Region::BoundsChangeInfo, void *);
    void region_muted ();
    void region_opacity ();
    void region_layered ();
    void region_renamed ();
    void region_sync_changed ();

    void reset_width_dependent_items (double pixel_width);

    void set_waveform_shape (WaveformShape);

    bool waveform_rectified() const { return _flags & WaveformRectified; }
    bool waveform_visible() const { return _flags & WaveformVisible; }
    bool envelope_visible() const { return _flags & EnvelopeVisible; }
    bool selected() const         { return _flags & Selected; }
    void set_selected (bool yn);
    
    void show_region_editor ();
    void hide_region_editor();

    void add_gain_point_event (GtkCanvasItem *item, GdkEvent *event);
    void remove_gain_point_event (GtkCanvasItem *item, GdkEvent *event);

    bool name_active() const { return name_connected; }

    static SigC::Signal1<void,RegionView*> GoingAway;


  private:
    RegionEditor *editor;
    vector<ControlPoint *> control_points;
    double _amplitude_above_axis;
    double samples_per_unit;
    double current_visible_sync_position;

    enum Flags {
	    EnvelopeVisible = 0x1,
	    Selected = 0x2,
	    WaveformVisible = 0x4,
	    WaveformRectified = 0x8
	    
    };

    unsigned int _flags;

    unsigned int wave_color;
    unsigned int muted_wave_color;
    unsigned int fill_color;
    unsigned int frame_color_r;
    unsigned int frame_color_g;
    unsigned int frame_color_b;
    unsigned int selected_frame_color_r;
    unsigned int selected_frame_color_g;
    unsigned int selected_frame_color_b;
    unsigned int label_color;
    unsigned int gain_color;
    bool     name_connected;

    void create_wave ();
    void compute_colors (GdkColor&);
    void set_colors ();
    void set_frame_color ();
    void peaks_ready_handler ();
    void reset_name (gdouble width);

    void set_flags (XMLNode *);
    void store_flags ();
};

#endif /* __gtk_ardour_region_view_h__ */
