/*
    Copyright (C) 2000 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: overlap.cc,v 1.2 2002/05/13 03:23:21 trutkin Exp $
*/

#include <ardour/overlap.h>
#include <ardour/region.h>

using namespace ARDOUR;

Overlap::Overlap (ARDOUR::Region *r, ARDOUR::Region *l)
	: right (r), left (l) 
{
	last_length = length();
}

Overlap::~Overlap ()
{
	right->use_default_fade_out ();
	left->use_default_fade_in ();
}

jack_nframes_t
Overlap::length () const
{
	return right->position() + right->length() - left->position();
}

void
Overlap::update ()
{
	jack_nframes_t newlen = length();

	if (newlen != last_length) {
		EMIT_SIGNAL Changed();
		last_length = newlen;
	}
}

bool
Overlap::Compare::operator() (const Overlap a, const Overlap b) const
{
	if (a.right == b.right && a.left == b.left) return 0;
	return &a < &b;
}

bool
Overlap::ComparePtr::operator() (const Overlap *a, const Overlap *b) const
{
	if (a->right == b->right && a->left == b->left) return 0;
	return a < b;
}
